#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/time.h>

static void processus_fils(void);
static void processus_pere(void);
static void handler_pere(int, siginfo_t *, void *);
static void handler_fils(int, siginfo_t *, void *);


int main(int argc,char * argv[])
{
	pid_t pid;
	pid = fork();
	if (pid < 0) {
		perror("fork");
		exit(EXIT_FAILURE);
	}

	if (pid == 0) {
		processus_fils();
	} else {
		processus_pere();
	}
	return EXIT_SUCCESS;
}


static void processus_fils(void)
{
	struct sigaction action;
	action.sa_flags = SA_SIGINFO;
	action.sa_sigaction = handler_fils;
	sigfillset(& action.sa_mask);
	sigaction(SIGRTMIN, & action, NULL);
	
	sleep(2); // Pour etre sur que le pere ait installe son handler
	raise(SIGRTMIN); // pour amorcer le ping-pong
	while (1)
		pause();
}


static void handler_fils(int unused, siginfo_t * info, void * unused_ptr)
{
	union sigval value;
	struct timeval heure;

	gettimeofday(& heure, NULL);
	value.sival_int = heure.tv_usec;
	sigqueue(getppid(), SIGRTMIN, value);
}


static void processus_pere(void)
{
	struct sigaction action;
	action.sa_flags = SA_SIGINFO;
	action.sa_sigaction = handler_pere;
	sigfillset(& action.sa_mask);
	sigaction(SIGRTMIN, & action, NULL);
	while (1)
		pause();
}


static void handler_pere(int unused, siginfo_t * info, void * unused_ptr)
{
	long int duree;
	static long int duree_max = 0;
	static long int duree_min = -1;
	static long int somme_durees = 0;
	static long int nb_recus = 0;
	static struct timeval debut = { 0, 0};
	struct timeval heure;
	
	gettimeofday(& heure, NULL);
	duree = heure.tv_usec - info->si_value.sival_int;
	if (duree < 0)
		duree += 1000000;

	if (duree_max < duree)
		duree_max = duree;
	if ((duree_min == -1) || (duree_min > duree))
		duree_min = duree;
	somme_durees += duree;

	nb_recus ++;
	if (heure.tv_sec != debut.tv_sec) {
		fprintf(stdout, "min =%3ld   max =%3ld  recus = %6ld  moy=%5.1f\n",
			duree_min, duree_max, nb_recus, ((float) somme_durees) / nb_recus);
		nb_recus = 0;
		duree_min = -1;
		duree_max = 0;
		somme_durees = 0;
		debut = heure;
	}
	sigqueue(info->si_pid, SIGRTMIN, (const union sigval) 0);
}


